@@ -6,7 +6,7 @@ use Carp;
 
 use Clone;
 use Rose::URI;
-use Scalar::Util qw(refaddr);
+use Scalar::Util();
 use URI::Escape qw(uri_escape);
 
 use Rose::HTML::Object::Errors qw(:form);
@@ -15,11 +15,12 @@ use Rose::HTML::Form::Field;
 use Rose::HTML::Form::Field::Collection;
 our @ISA = qw(Rose::HTML::Form::Field Rose::HTML::Form::Field::Collection);
 
-our $VERSION = '0.547';
+our $VERSION = '0.551';
 
 # Multiple inheritence never quite works out the way I want it to...
 Rose::HTML::Form::Field::Collection->import_methods
 (
+  'prepare',
   'children',
   'hidden_field',
   'hidden_fields',
@@ -39,6 +40,7 @@ __PACKAGE__->add_valid_html_attrs
   'onsubmit',       # %Script;       #IMPLIED  -- the form was submitted --
   'onreset',        # %Script;       #IMPLIED  -- the form was reset --
   'accept-charset', # %Charsets;     #IMPLIED  -- list of supported charsets --
+  'target',         # http://www.w3.org/TR/xhtml-modularization/abstract_modules.html#s_targetmodule
 );
 
 __PACKAGE__->add_required_html_attrs(
@@ -674,7 +676,7 @@ sub _init_field
   $Debug && warn "INIT FIELD $name ($name_attr)\n";
 
   my $name_exists       = $self->param_exists($name);
-  my $moniker_exists = $self->param_exists($moniker);
+  my $moniker_exists    = $self->param_exists($moniker);
   my $name_attr_exists  = $self->param_exists($name_attr);
 
   if(!$name_exists && $field->isa('Rose::HTML::Form::Field::Compound'))
@@ -712,7 +714,8 @@ sub _init_field
       # Must handle lone checkboxes and radio buttons here
       if($on_off)
       {
-        if($self->param($name) eq $field->html_attr('value'))
+        no warnings 'uninitialized';
+        if($name_exists && $self->param($name) eq $field->html_attr('value'))
         {
           $Debug && warn "$self->param($name) = checked\n";
           $field->checked(1);
@@ -762,6 +765,18 @@ sub _init_field
   }
 }
 
+sub was_submitted
+{
+  my($self) = shift;
+
+  foreach my $field ($self->fields)
+  {
+    return 1  if($self->param_exists_for_field($field->name));
+  }
+
+  return 0;
+}
+
 sub start_html
 {
   my($self) = shift;
@@ -922,7 +937,7 @@ sub add_forms
     {
       $form = $arg;
 
-      if(refaddr($form) eq refaddr($self))
+      if(Scalar::Util::refaddr($form) eq Scalar::Util::refaddr($self))
       {
         croak "Cannot nest a form within itself";
       }
@@ -947,7 +962,7 @@ sub add_forms
 
       if(UNIVERSAL::isa($form, 'Rose::HTML::Form'))
       {
-        if(refaddr($form) eq refaddr($self))
+        if(Scalar::Util::refaddr($form) eq Scalar::Util::refaddr($self))
         {
           croak "Cannot nest a form within itself";
         }
@@ -1394,6 +1409,20 @@ sub form
   return $parent_form->form($local_name);
 }
 
+sub app
+{
+  my($self) = shift; 
+  Scalar::Util::weaken($self->{'app'} = shift)  if(@_);
+  return $self->{'app'};
+}
+
+sub app_form
+{
+  my($self) = shift; 
+  Scalar::Util::weaken($self->{'app_form'} = shift)  if(@_);
+  return $self->{'app_form'};
+}
+
 our $AUTOLOAD;
 
 sub AUTOLOAD
@@ -1425,6 +1454,7 @@ sub AUTOLOAD
 
   unless($to_form)
   {
+    $Rose::HTML::Object::AUTOLOAD = $AUTOLOAD;
     goto &Rose::HTML::Object::AUTOLOAD;
   }
 
@@ -1442,6 +1472,7 @@ sub AUTOLOAD
     $to_form->$method(@_);
   }
 
+  $Rose::HTML::Object::AUTOLOAD = $AUTOLOAD;
   goto &Rose::HTML::Object::AUTOLOAD;
 }
 
@@ -1466,6 +1497,10 @@ FORM_HAS_ERRORS = "Ein oder mehrere Felder sind fehlerhaft."
 
 FORM_HAS_ERRORS = "Erreurs dans un ou plusieurs champs."
 
+[% LOCALE bg %]
+
+FORM_HAS_ERRORS = "Има грешка в поне едно поле."
+
 __END__
 
 =head1 NAME
@@ -1715,6 +1750,7 @@ Valid attributes:
     onsubmit
     style
     tabindex
+    target
     title
     value
     xml:lang
@@ -1907,6 +1943,10 @@ Add VALUE to the parameter named NAME.  Example:
 
     print join(',', $form->param('a')); # 1,2
 
+=item B<app [OBJECT]>
+
+Get or set a L<weakened|Scalar::Util/weaken> reference to the application object that "owns" this form.
+
 =item B<build_on_init [BOOL]>
 
 Get or set a boolean flag that indicates whether or not L<build_form()|/build_form> should be called from within the L<init()|Rose::Object/init> method.  See L<build_form()|/build_form> for more information.
@@ -2103,6 +2143,10 @@ The default mapping of type names to class names is:
 
   'hidden'             => Rose::HTML::Form::Field::Hidden
 
+  'num'                => Rose::HTML::Form::Field::Numeric,
+  'number'             => Rose::HTML::Form::Field::Numeric,
+  'numeric'            => Rose::HTML::Form::Field::Numeric,
+
   'int'                => Rose::HTML::Form::Field::Integer,
   'integer'            => Rose::HTML::Form::Field::Integer,
 
@@ -2139,16 +2183,26 @@ The default mapping of type names to class names is:
   'datetime split mdyhms' => 
     Rose::HTML::Form::Field::DateTime::Split::MDYHMS
 
-=item B<field_value NAME>
+=item B<field_value NAME [, VALUE]>
 
-Returns the L<internal_value|Rose::HTML::Form::Field/internal_value> of the field named NAME.  In other words, this:
+If passed NAME and VALUE arguments, then the L<input_value|Rose::HTML::Form::Field/input_value> of the field named NAME is set to VALUE.  If passed only a NAME, then the L<internal_value|Rose::HTML::Form::Field/internal_value> of the field named NAME is returned.  In other words, this:
+
+    $form->field_value(zip_code => '11787');
+
+is equivalent to this:
+
+    $form->field('zip_code')->input_value('11787');
+
+and this:
 
     $val = $form->field_value('zip_code');
 
-is just a shorter way to write this:
+is equivalent to this:
 
     $val = $form->field('zip_code')->internal_value;
 
+If no field named NAME exists, a fatal error will occur.
+
 =item B<form NAME [, OBJECT]>
 
 Get or set the sub-form named NAME.  If just NAME is passed, the specified sub-form object is returned.  If no such sub-form exists, undef is returnend.
@@ -2533,6 +2587,20 @@ A fatal error occurs unless both NAME and VALUE arguments are passed.
 
 Get or set the parent form, if any.  The reference to the parent form is "weakened" using L<Scalar::Util::weaken()|Scalar::Util/weaken> in order to avoid memory leaks caused by circular references.
 
+=item B<prepare [PARAMS]>
+
+Prepare the form by calling the L<prepare|Rose::HTML::Form::Field/prepare> method on each L<field|/fields>, passing all arguments.  This same method is also called for each L<sub-form|/forms>, passing all arguments plus the C<form_only> parameter with a true value.
+
+PARAMS are name/value pairs.  Valid parameters are:
+
+=over 4
+
+=item B<form_only BOOL>
+
+If true, the L<prepare|Rose::HTML::Form::Field/prepare> method is not called for each  L<field|/fields>.
+
+=back
+
 =item B<query_string>
 
 Returns a URI-escaped (but I<not> HTML-escaped) query string that corresponds to the current state of the form.  If L<coalesce_query_string_params()|/coalesce_query_string_params> is true (which is the default), then compound fields are represented by a single query parameter.  Otherwise, the subfields of each compound field appear as separate query parameters.
@@ -2633,6 +2701,10 @@ Examples:
 
 Get or set a boolean flag that indicates whether or not the fields of this form will validate their HTML attributes.  If a BOOL argument is passed, then it is passed as the argument to a call to L<validate_html_attrs()|Rose::HTML::Object/validate_html_attrs> on each field.  In either case, the current value of this flag is returned.
 
+=item B<was_submitted>
+
+Returns true id L<params exist|/param_exists_for_field> for any L<field|/fields>, false otherwise.
+
 =item B<xhtml_hidden_fields>
 
 Returns the XHTML serialization of the fields returned by L<hidden_fields()|/hidden_fields>, joined by newlines.
@@ -2651,7 +2723,7 @@ L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Rose-HTML-Objects>
 
 =head1 AUTHOR
 
-John C. Siracusa (siracusa@mindspring.com)
+John C. Siracusa (siracusa@gmail.com)
 
 =head1 COPYRIGHT
 
